﻿using System.Windows;
using IoFireBugEngine;
using System.Windows.Threading;
using System;
using System.Windows.Media;


namespace IOFireBug_sample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        IoFireBugDeviceV1 Device;
        IoFireBugComm Comm;

        const int baundrate = 115200;
        const int address = 1;

        byte[] serial = null;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void IoFireBugInitialize()
        {
            Device = new IoFireBugDeviceV1();
            Device.Control.Addr = address;

            Device.Control.BinInputsFnc = (byte)IoFireBugConst.BIN_INPUT_FUNCTION.ON_CHANGE;
            Device.Control.AnalogInputsFnc = (byte)IoFireBugConst.ADC_FUNCTION.ON_PER_50MS;
            Device.Control.RotaryCountersFnc = (byte)IoFireBugConst.ROTARY_COUNTERS_FUNCTION.ON_CHANGE;

            Device.Config.FtdiBaudrate = baundrate;
            Device.Config.BinInputsFilterLen = 4;
            Device.Config.BinInputsSamplePeriod = 4;

            Device.Config.AdcEnable = true;
            Device.Config.AdcFreqHz = 1000000;
            Device.Config.AdcInputFrom = 0;
            Device.Config.AdcInputTo = 7;
            Device.Config.AdcAverages = 1;
            Device.Config.AdcRightZeros = 0;
            Device.Config.AdcResolution = IoFireBugConst.ADC_RESOLUTION.ADC_12BIT;
            Device.Config.AdcMultiplier = 1;

            if (serial == null)
            {
                MessageBox.Show("Invalid serial number!");
                return;
            }

            Device.Control.Serial = serial;

            Comm = new IoFireBugComm("COM4", baundrate, 200);
            Comm.AddDevice(Device);
            Comm.Start();
            
            GuiState(true);

            Device.Data.PropertyChanged += IoFireBugData_Changed;
        }

        private void IoFireBugClose()
        {
            if (Comm != null)
            {
                Comm.Close();
                Comm = null;
            }

            if (Device != null)
            {
                Device.Stop();
                Device.Data.PropertyChanged -= IoFireBugData_Changed;
                Device = null;
            }

            GuiState(false);
        }

        private void IoFireBugData_Changed(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
                Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(() =>
                {
                    if (Comm != null && Device != null)
                    {
                        if (e.PropertyName == "Inputs")
                        {
                            checkBoxDigitalInput1.IsChecked = Device.Data.Inputs[0];
                            checkBoxDigitalInput2.IsChecked = Device.Data.Inputs[1];
                            checkBoxDigitalInput3.IsChecked = Device.Data.Inputs[2];
                            checkBoxDigitalInput4.IsChecked = Device.Data.Inputs[3];
                            checkBoxDigitalInput5.IsChecked = Device.Data.Inputs[4];
                            checkBoxDigitalInput6.IsChecked = Device.Data.Inputs[5];
                            checkBoxDigitalInput7.IsChecked = Device.Data.Inputs[6];
                            checkBoxDigitalInput8.IsChecked = Device.Data.Inputs[7];
                        }

                        if (e.PropertyName == "AdcValues")
                        {
                            lblAnalogInput9.Content = Device.Data.AdcValues[0];
                            lblAnalogInput10.Content = Device.Data.AdcValues[1];
                            lblAnalogInput11.Content = Device.Data.AdcValues[2];
                            lblAnalogInput12.Content = Device.Data.AdcValues[3];
                            lblAnalogInput13.Content = Device.Data.AdcValues[4];
                            lblAnalogInput14.Content = Device.Data.AdcValues[5];
                            lblAnalogInput15.Content = Device.Data.AdcValues[6];
                            lblAnalogInput16.Content = Device.Data.AdcValues[7];
                        }
                        if (e.PropertyName == "RotaryCounters")
                        {
                            lblRotaryEncoder5.Content = Device.Data.RotaryCounters[0];
                            lblRotaryEncoder6.Content = Device.Data.RotaryCounters[1];
                            lblRotaryEncoder7.Content = Device.Data.RotaryCounters[2];
                            lblRotaryEncoder8.Content = Device.Data.RotaryCounters[3];
                        }
                    }
                }));
        }

        private void checkBoxDigitalOutput_Changed(object sender, RoutedEventArgs e)
        {
            Comm.GetDeviceData(address).Outputs = new bool[] 
            {
                (bool)checkBoxDigitalOutput1.IsChecked,
                (bool)checkBoxDigitalOutput2.IsChecked,
                (bool)checkBoxDigitalOutput3.IsChecked,
                (bool)checkBoxDigitalOutput4.IsChecked,
                (bool)checkBoxDigitalOutput5.IsChecked,
                (bool)checkBoxDigitalOutput6.IsChecked,
                (bool)checkBoxDigitalOutput7.IsChecked,
                (bool)checkBoxDigitalOutput8.IsChecked,
            };
        }

        private void GuiState(bool communicationOpen)
        {
            grpBoxDigitalOutputs.IsEnabled = communicationOpen;
            grpBoxDigitalInputs.IsEnabled = communicationOpen;
            grpBoxAnalogInputs.IsEnabled = communicationOpen;
            grpRotaryEncoders.IsEnabled = communicationOpen;

            btnOpen.IsEnabled = !communicationOpen;
            btnClose.IsEnabled = communicationOpen;
        }

        private void btnOpen_Click(object sender, RoutedEventArgs e)
        {
            IoFireBugInitialize();
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            IoFireBugClose();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            IoFireBugClose();
        }

        private void txbSerial_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            try
            {
                string[] valsStr = txbSerial.Text.Split('-');
                if (valsStr.Length == IoFireBugConst.SERIAL_NUMBER_LEN)
                {
                    serial = new byte[IoFireBugConst.SERIAL_NUMBER_LEN];
                    for (int i = 0; i < IoFireBugConst.SERIAL_NUMBER_LEN; i++)
                    {
                        serial[i] = byte.Parse(valsStr[i], System.Globalization.NumberStyles.HexNumber);
                    }
                    txbSerial.Background = Brushes.Green;
                }
                else
                {
                    throw new Exception("invalid serial number");
                }
            }
            catch (Exception)
            {
                serial = null;
                txbSerial.Background = Brushes.Red;
            }
        }
    }
}
